"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useControllableState = useControllableState;
exports.useOn = exports.usePopover = exports.useElementByType = exports.useKeyboardDismissable = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let idCounter = 0;

const generateId = prefix => {
  idCounter++;
  return 'react-native-popper-' + prefix + '-' + idCounter;
};

const useKeyboardDismissable = ({
  enabled,
  onClose
}) => {
  _react.default.useEffect(function closeOverlayOnEscapeEffectCallback() {
    let escapeKeyListener = null;

    if (_reactNative.Platform.OS === 'web') {
      escapeKeyListener = e => {
        if (e.key === 'Escape' && enabled) {
          onClose();
        }
      };

      document.addEventListener('keydown', escapeKeyListener);
    }

    return () => {
      if (_reactNative.Platform.OS === 'web') {
        document.removeEventListener('keydown', escapeKeyListener);
      }
    };
  }, [enabled, onClose]);
};

exports.useKeyboardDismissable = useKeyboardDismissable;

const useElementByType = (children, name) => {
  let element;

  _react.default.Children.forEach(children, child => {
    //@ts-ignore
    if (child.type.displayName === name) {
      element = child;
    }
  });

  return element;
};

exports.useElementByType = useElementByType;

/**
 * React hook for using controlling component state.
 * @param props
 */
function useControllableState(props) {
  const {
    value: valueProp,
    defaultValue,
    onChange
  } = props;

  const [valueState, setValue] = _react.default.useState(defaultValue);

  const isControlled = valueProp !== undefined;
  const value = isControlled ? valueProp : valueState;

  const updateValue = _react.default.useCallback(next => {
    const nextValue = typeof next === 'function' ? next(value) : next;

    if (!isControlled) {
      setValue(nextValue);
    }

    onChange && onChange(nextValue);
  }, [isControlled, onChange, value]);

  return [value, updateValue];
}

const usePopover = props => {
  const triggerId = _react.default.useRef(generateId('trigger')).current;

  const contentId = _react.default.useRef(generateId('content')).current;

  const {
    isOpen,
    mode
  } = props;
  let triggerProps = {
    nativeID: triggerId,
    accessibilityRole: 'button'
  };
  let contentProps = {
    nativeID: contentId
  };

  if (mode === 'tooltip') {
    triggerProps['aria-describedby'] = isOpen ? contentId : undefined;
    contentProps.accessibilityRole = 'tooltip';
  }

  if (mode === 'popover') {
    triggerProps['aria-expanded'] = !!isOpen;
    triggerProps['aria-haspopup'] = true;
    triggerProps['aria-controls'] = isOpen ? contentId : undefined;
    contentProps.accessibilityRole = 'dialog';
  }

  return {
    triggerProps,
    contentProps
  };
}; // export const useHoverOrFocus = () => {
//   const [isFocused, setIsFocused] = React.useState(false);
//   const [isHovered, setIsHovered] = React.useState(false);
//   const onHoverIn = React.useCallback(() => {
//     setIsHovered(true);
//   }, [setIsHovered]);
//   const onHoverOut = React.useCallback(() => {
//     setIsHovered(false);
//   }, [setIsHovered]);
//   const onFocus = React.useCallback(() => {
//     setIsFocused(true);
//   }, [setIsFocused]);
//   const onBlur = React.useCallback(() => {
//     setIsFocused(false);
//   }, [setIsFocused]);
//   return {
//     triggerProps: {
//       onHoverIn,
//       onHoverOut,
//       onFocus,
//       onBlur,
//     },
//     isHovered,
//     isFocused,
//   };
// };
// type IUseHoverTrigger = {
//   onOpen: () => void;
//   onClose: () => void;
//   hoverTimeout?: number;
//   isOpen: boolean;
// };
// export const useHoverTrigger = (props: IUseHoverTrigger) => {
//   const { triggerProps, isHovered, isFocused } = useHoverOrFocus();
//   let hoverTimeoutHandler = React.useRef<any>(null);
//   const { onOpen, onClose, isOpen, hoverTimeout = 0 } = props;
//   React.useEffect(() => {
//     // Immediately open popover on focus.
//     if (isFocused) {
//       props.onOpen();
//     } else if (!isHovered) {
//       // Immediately close on blur
//       onClose();
//     }
//   }, [isFocused]);
//   React.useEffect(() => {
//     if (isHovered) {
//       hoverTimeoutHandler.current = setTimeout(() => {
//         onOpen();
//       }, hoverTimeout);
//     } else {
//       if (isOpen) {
//         onClose();
//       }
//       // Immediately close on blur
//       clearTimeout(hoverTimeoutHandler.current);
//     }
//   }, [isHovered]);
//   return {
//     triggerProps,
//   };
// };


exports.usePopover = usePopover;

const useOn = props => {
  const {
    on = 'press',
    onOpen,
    onClose
  } = props;

  if (on === 'press') {
    return {
      onPress: (0, _utils.composeEventHandlers)(props.onPress, () => {
        onOpen();
      })
    };
  }

  if (on === 'longPress') {
    return {
      onLongPress: (0, _utils.composeEventHandlers)(props.onLongPress, () => {
        onOpen();
      })
    };
  }

  if (on === 'hover') {
    return {
      onHoverIn: (0, _utils.composeEventHandlers)(props.onHoverIn, () => {
        onOpen();
      }),
      onHoverOut: (0, _utils.composeEventHandlers)(props.onHoverOut, () => {
        onClose();
      }),
      onFocus: (0, _utils.composeEventHandlers)(props.onFocus, () => {
        onOpen();
      }),
      onBlur: (0, _utils.composeEventHandlers)(props.onBlur, () => {
        onClose();
      })
    };
  }

  return {};
};

exports.useOn = useOn;
//# sourceMappingURL=index.js.map