import React from 'react';
import { Platform } from 'react-native';
import { composeEventHandlers } from '../utils';
let idCounter = 0;

const generateId = prefix => {
  idCounter++;
  return 'react-native-popper-' + prefix + '-' + idCounter;
};

export const useKeyboardDismissable = ({
  enabled,
  onClose
}) => {
  React.useEffect(function closeOverlayOnEscapeEffectCallback() {
    let escapeKeyListener = null;

    if (Platform.OS === 'web') {
      escapeKeyListener = e => {
        if (e.key === 'Escape' && enabled) {
          onClose();
        }
      };

      document.addEventListener('keydown', escapeKeyListener);
    }

    return () => {
      if (Platform.OS === 'web') {
        document.removeEventListener('keydown', escapeKeyListener);
      }
    };
  }, [enabled, onClose]);
};
export const useElementByType = (children, name) => {
  let element;
  React.Children.forEach(children, child => {
    //@ts-ignore
    if (child.type.displayName === name) {
      element = child;
    }
  });
  return element;
};

/**
 * React hook for using controlling component state.
 * @param props
 */
export function useControllableState(props) {
  const {
    value: valueProp,
    defaultValue,
    onChange
  } = props;
  const [valueState, setValue] = React.useState(defaultValue);
  const isControlled = valueProp !== undefined;
  const value = isControlled ? valueProp : valueState;
  const updateValue = React.useCallback(next => {
    const nextValue = typeof next === 'function' ? next(value) : next;

    if (!isControlled) {
      setValue(nextValue);
    }

    onChange && onChange(nextValue);
  }, [isControlled, onChange, value]);
  return [value, updateValue];
}
export const usePopover = props => {
  const triggerId = React.useRef(generateId('trigger')).current;
  const contentId = React.useRef(generateId('content')).current;
  const {
    isOpen,
    mode
  } = props;
  let triggerProps = {
    nativeID: triggerId,
    accessibilityRole: 'button'
  };
  let contentProps = {
    nativeID: contentId
  };

  if (mode === 'tooltip') {
    triggerProps['aria-describedby'] = isOpen ? contentId : undefined;
    contentProps.accessibilityRole = 'tooltip';
  }

  if (mode === 'popover') {
    triggerProps['aria-expanded'] = !!isOpen;
    triggerProps['aria-haspopup'] = true;
    triggerProps['aria-controls'] = isOpen ? contentId : undefined;
    contentProps.accessibilityRole = 'dialog';
  }

  return {
    triggerProps,
    contentProps
  };
}; // export const useHoverOrFocus = () => {
//   const [isFocused, setIsFocused] = React.useState(false);
//   const [isHovered, setIsHovered] = React.useState(false);
//   const onHoverIn = React.useCallback(() => {
//     setIsHovered(true);
//   }, [setIsHovered]);
//   const onHoverOut = React.useCallback(() => {
//     setIsHovered(false);
//   }, [setIsHovered]);
//   const onFocus = React.useCallback(() => {
//     setIsFocused(true);
//   }, [setIsFocused]);
//   const onBlur = React.useCallback(() => {
//     setIsFocused(false);
//   }, [setIsFocused]);
//   return {
//     triggerProps: {
//       onHoverIn,
//       onHoverOut,
//       onFocus,
//       onBlur,
//     },
//     isHovered,
//     isFocused,
//   };
// };
// type IUseHoverTrigger = {
//   onOpen: () => void;
//   onClose: () => void;
//   hoverTimeout?: number;
//   isOpen: boolean;
// };
// export const useHoverTrigger = (props: IUseHoverTrigger) => {
//   const { triggerProps, isHovered, isFocused } = useHoverOrFocus();
//   let hoverTimeoutHandler = React.useRef<any>(null);
//   const { onOpen, onClose, isOpen, hoverTimeout = 0 } = props;
//   React.useEffect(() => {
//     // Immediately open popover on focus.
//     if (isFocused) {
//       props.onOpen();
//     } else if (!isHovered) {
//       // Immediately close on blur
//       onClose();
//     }
//   }, [isFocused]);
//   React.useEffect(() => {
//     if (isHovered) {
//       hoverTimeoutHandler.current = setTimeout(() => {
//         onOpen();
//       }, hoverTimeout);
//     } else {
//       if (isOpen) {
//         onClose();
//       }
//       // Immediately close on blur
//       clearTimeout(hoverTimeoutHandler.current);
//     }
//   }, [isHovered]);
//   return {
//     triggerProps,
//   };
// };

export const useOn = props => {
  const {
    on = 'press',
    onOpen,
    onClose
  } = props;

  if (on === 'press') {
    return {
      onPress: composeEventHandlers(props.onPress, () => {
        onOpen();
      })
    };
  }

  if (on === 'longPress') {
    return {
      onLongPress: composeEventHandlers(props.onLongPress, () => {
        onOpen();
      })
    };
  }

  if (on === 'hover') {
    return {
      onHoverIn: composeEventHandlers(props.onHoverIn, () => {
        onOpen();
      }),
      onHoverOut: composeEventHandlers(props.onHoverOut, () => {
        onClose();
      }),
      onFocus: composeEventHandlers(props.onFocus, () => {
        onOpen();
      }),
      onBlur: composeEventHandlers(props.onBlur, () => {
        onClose();
      })
    };
  }

  return {};
};
//# sourceMappingURL=index.js.map